package formData;

import java.lang.reflect.Method;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import javax.servlet.http.HttpServletRequest;

/**
 * The FormDataBuilder is used to get the parameters from an HTTP request and
 * and fills an associated FormData object with it's values. The builder assumes
 * that there is a setter in the FormData object having the same name as that of
 * the parameters in the request object.
 * 
 * @author Paul Salvador Inventado <paul.inventado@delasalle.ph>
 * 
 */
public class FormDataBuilder {

	/**
	 * Disables instantiation of the object
	 */
	private FormDataBuilder() {

	}

	/**
	 * Constructs a FormDataBuilder given the request object containing
	 * parameters from a form and the FormData object which will contain these
	 * values.
	 * 
	 * @param request
	 *            object containing parameter values from a form
	 * @param obj
	 *            FormData object which will contain the parameter values
	 * @return FormData object
	 */
	public static FormData build(HttpServletRequest request, FormData obj) {
		
		//Retrieves all getter methods from the FormData object
		HashMap<String, Method> methodList = new HashMap<String, Method>();
		for (Method mtd : obj.getClass().getMethods()) {
			methodList.put(mtd.getName(), mtd);
		}

		//Retrieves all parameters passed in the request object
		Enumeration<String> parameters = request.getParameterNames();
		while (parameters.hasMoreElements()) {
			String parameter = parameters.nextElement();
			String capitalized = Character.toUpperCase(parameter.charAt(0))
					+ parameter.substring(1);
			Class[] parameterTypes;
			if (methodList.containsKey("set" + capitalized)) {
				parameterTypes = methodList.get("set" + capitalized)
						.getParameterTypes();
				String type = parameterTypes[0].getSimpleName();
				//invokes the setter method of the FormData and passes as a parameter 
				//the value of the form stored in the request object 
				try {
					if (type.equals("String")) {
						methodList.get("set" + capitalized).invoke(obj,
								request.getParameter(parameter));
					} else if (type.equals("char")) {
						methodList.get("set" + capitalized).invoke(obj,
								request.getParameter(parameter).charAt(0));
					} else if (type.equals("long")) {
						methodList.get("set" + capitalized)
								.invoke(
										obj,
										Long.parseLong(request
												.getParameter(parameter)));
					} else if (type.equals("double")) {
						methodList.get("set" + capitalized).invoke(
								obj,
								Double.parseDouble(request
										.getParameter(parameter)));
					} else if (type.equals("Date")) {
						methodList.get("set" + capitalized).invoke(
								obj,
								new Date(Long.parseLong(request
										.getParameter(parameter))));
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		return obj;
	}
}